/*
 *	PICCLIB.h
 *	PHOTRON Image Control SDK (Compact version)
 *
 *	Copyright (C) 2014 PHOTRON LIMITED
 */

#ifndef	_PICLIB_H_
#define	_PICLIB_H_

#include	<vfw.h>

#ifdef	__cplusplus
extern "C" {
#endif	/*	__cplusplus	*/

/*	Return Value	*/
#define	PIC_SUCCEEDED				1	/*		*/
#define	PIC_FAILED					0	/*	s	*/

/*	Error code	*/
#define	PIC_ERROR_NOERROR			1	/*		*/
#define	PIC_ERROR_UNINITIALIZE		2	/*		*/
#define	PIC_ERROR_ILLEGAL_VALUE		3	/*	p[^s	*/
#define	PIC_ERROR_LIBRARY_ERROR		4	/*	Cuُ	*/
#define	PIC_ERROR_INITIALIZED		6	/*	ɏς	*/
#define	PIC_ERROR_ILLEGAL_STRUCTURE	7	/*	\̂s	*/
#define	PIC_ERROR_AVI_NOT_OPENED	8	/*	AVI쐬I[vĂȂ	*/
#define	PIC_ERROR_TIMEOUT			9	/*	^CAEg	*/
#define	PIC_ERROR_FILEWRITE_FAILED	12	/*	݃G[	*/
#define PIC_ERROR_FILEREAD_FAILED	13	/*  ǂݍ݃G[  */
#define	PIC_ERROR_FILEOPENED		16	/*	I[vς	*/
#define	PIC_ERROR_ILLEGAL_DATA		17	/*	ݒf[^s	*/
#define	PIC_ERROR_FILE_EXIST		18	/*	ɂ̃t@C݂Ă	*/
#define	PIC_ERROR_NOT_SUPPORTED		19	/*	T|[g	*/
#define	PIC_ERROR_CONVERSION_OF_STRING	25	/*	Conversion of string. Multi-byte and Unicode */
#define	PIC_ERROR_NOT_REGISTERED	30	/*	MPEGIvṼCZXȂ	*/
#define	PIC_ERROR_MPEG_NOT_OPENED	31	/*	MPEG쐬I[vĂȂ	*/
#define PIC_ERROR_FILEREAD_NOT_SUPPORTED	40	/*	ΉĂȂt@C\ (pbg`)	*/
#define PIC_ERROR_FILEREAD_BUFFSIZE	41	/*	\̂ɋLڂĂ摜obt@TCYt@Cwb_ɋLڂĂTCYƈvȂ	*/

/*	Function On/Off(General)	*/
#define	PIC_FUNCTION_OFF			0
#define	PIC_FUNCTION_ON				1

/*	Image Buffer Information	*/
typedef struct {
	long			m_nWidth;
	long			m_nHeight;
	long			m_nBytePerLine;
	unsigned short	m_nRealBitPerPixel;
	long			m_nInterleave;	/* Buffer Interleave	*/
} PIC_IMAGE_BUFF_INFO, *PPIC_IMAGE_BUFF_INFO;

/*	Bayer Convert Information	*/
typedef struct {
	unsigned long	m_nMode;
	unsigned long	m_nDeviceCode;
	void*			m_pLUTR;
	void*			m_pLUTG;
	void*			m_pLUTB;
	unsigned long	m_nOutputInterleave;
	unsigned long	m_nEdgeEnhance;
	unsigned long	m_nColorTempR;
	unsigned long	m_nColorTempG;
	unsigned long	m_nColorTempB;
	long			m_nColorMatrixRR;
	long			m_nColorMatrixRG;
	long			m_nColorMatrixRB;
	long			m_nColorMatrixGR;
	long			m_nColorMatrixGG;
	long			m_nColorMatrixGB;
	long			m_nColorMatrixBR;
	long			m_nColorMatrixBG;
	long			m_nColorMatrixBB;
	unsigned long	m_nBlackClip;
} PIC_BAYER_CONVERT_INFO, *PPIC_BAYER_CONVERT_INFO;

/* 
 * Original Image data Information
 */
typedef struct {
	unsigned long	m_nDeviceCode;
	unsigned long	m_nColorType;
	unsigned short	m_nBitDepth;
	unsigned long	m_nEffectiveBits;
	unsigned long	m_n8BitSel;
} PIC_ORG_IMAGE_INFO, *PPIC_ORG_IMAGE_INFO;

/*	Data Interleave	*/
#define	PIC_COLORDATA_NOCOLOR				0
#define	PIC_COLORDATA_INTERLEAVE_BGR		1
#define	PIC_COLORDATA_INTERLEAVE_RGB		2

/*	Effective Bit	*/
#define	PIC_EFFECTIVE_BITS_HIGH				0
#define	PIC_EFFECTIVE_BITS_LOW				1

/*	TIFF Compress Mode	*/
#define	PIC_TIFF_COMPRESS_NONE				0
#define	PIC_TIFF_COMPRESS_PACKBITS			1

/*	BayerConvert	*/
#define	PIC_BAYERCONVERT_MODE1				0
#define	PIC_BAYERCONVERT_MODE2				1
#define	PIC_BAYERCONVERT_MODE3				2

/*	EdgeEnhance	*/
#define	PIC_EDGE_ENHANCE_OFF				1
#define	PIC_EDGE_ENHANCE_MODE1				2
#define	PIC_EDGE_ENHANCE_MODE2				3
#define	PIC_EDGE_ENHANCE_MODE3				4

/*	BlackClip	*/
#define	PIC_BLACK_CLIP_LEVEL_OFF			0
#define	PIC_BLACK_CLIP_LEVEL_1				1
#define	PIC_BLACK_CLIP_LEVEL_2				2
#define	PIC_BLACK_CLIP_LEVEL_3				3
#define	PIC_BLACK_CLIP_LEVEL_4				4
#define	PIC_BLACK_CLIP_LEVEL_5				5
#define	PIC_BLACK_CLIP_LEVEL_6				6
#define	PIC_BLACK_CLIP_LEVEL_7				7
#define	PIC_BLACK_CLIP_LEVEL_8				8


/* CPU߃Zbg󂯂邽߂̌^ */
typedef UINT32	PIC_INSTRUCTIONSET;

/* CPU߃Zbg̎ʎq */
// ŏ2bit͎w肵߃ZbgΉƂ̃ftHg̎wtO
// cbit͖߃Zbg̎wtO
#define PIC_INSTSET_NONE	0x80000000
#define PIC_INSTSET_AUTO	0x40000000
#define PIC_INSTSET_MMX		0x00000001
#define PIC_INSTSET_SSE		0x00000002
#define PIC_INSTSET_SSE2	0x00000004
#define PIC_INSTSET_AVX2	0x00000008
#define PIC_INSTSET_3DNOW	0x00000010
#define PIC_INSTSET_E3DNOW	0x00000020

typedef struct
{
	void				*tiff_info;
	PIC_IMAGE_BUFF_INFO	image_info;
	int					num_page;
} PIC_MTIFF;

typedef struct
{
	void				*raw_info;
	PIC_IMAGE_BUFF_INFO	image_info;
	int					num_page;
} PIC_MRAW;

typedef struct
{
	void				*vid_info;
	PIC_IMAGE_BUFF_INFO	image_info;
	BOOL				bMultiHead;
	int					total_frame;
	PBYTE				m_pLUTR;
	PBYTE				m_pLUTG;
	PBYTE				m_pLUTB;
} PIC_VID;

typedef enum tag_PIC_TIFF_BITDEPTH
{
	//PIC_TIFF_BITDEPTH_8,
	PIC_TIFF_BITDEPTH_10,
	PIC_TIFF_BITDEPTH_12,
	PIC_TIFF_BITDEPTH_16,
} PIC_TIFF_BITDEPTH;

typedef enum tag_PIC_RAW_BITDEPTH
{
	PIC_RAW_BITDEPTH_8,
	PIC_RAW_BITDEPTH_10,
	PIC_RAW_BITDEPTH_12,
	PIC_RAW_BITDEPTH_16,
} PIC_RAW_BITDEPTH;

/*
 *	File Save
 */
unsigned long WINAPI
PIC_Init(
		unsigned long			*pErrorCode
		);


unsigned long WINAPI
PIC_RAWFileSaveA(
		LPCSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		unsigned char			*pBuff,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_RAWFileSaveW(
		LPCWSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		unsigned char			*pBuff,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_RAWFileSave  PIC_RAWFileSaveW
#else
#define PIC_RAWFileSave  PIC_RAWFileSaveA
#endif // !UNICODE

unsigned long WINAPI
PIC_RAW16FileSaveA(
		LPCSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		unsigned short			*pBuff,
		unsigned long			nEffectiveBit,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_RAW16FileSaveW(
		LPCWSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		unsigned short			*pBuff,
		unsigned long			nEffectiveBit,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_RAW16FileSave  PIC_RAW16FileSaveW
#else
#define PIC_RAW16FileSave  PIC_RAW16FileSaveA
#endif // !UNICODE

unsigned long WINAPI
PIC_MRAWFileSaveOpenA(
		LPCSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		long					nMaxFrameNum,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MRAWFileSaveOpenW(
		LPCWSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		long					nMaxFrameNum,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_MRAWFileSaveOpen  PIC_MRAWFileSaveOpenW
#else
#define PIC_MRAWFileSaveOpen  PIC_MRAWFileSaveOpenA
#endif // !UNICODE

unsigned long WINAPI
PIC_MRAWFileSaveOpenWithA(
		PIC_MRAW				*pPicMRaw,
		LPCSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		long					nMaxFrameNum,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MRAWFileSaveOpenWithW(
		PIC_MRAW				*pPicMRaw,
		LPCWSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		long					nMaxFrameNum,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_MRAWFileSaveOpenWith  PIC_MRAWFileSaveOpenWithW
#else
#define PIC_MRAWFileSaveOpenWith  PIC_MRAWFileSaveOpenWithA
#endif // !UNICODE

unsigned long WINAPI
PIC_MRAWFileSave(
		unsigned char			*pBuff,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MRAWFileSaveWith(
		PIC_MRAW				*pPicMRaw,
		unsigned char			*pBuff,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI 
PIC_MRAW16FileSaveOpenA(
		LPCSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		PIC_RAW_BITDEPTH		pic_depth,
		unsigned long			nEffectiveBit,
		long					nMaxFrameNum,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MRAW16FileSaveOpenW(
		LPCWSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		PIC_RAW_BITDEPTH		pic_depth,
		unsigned long			nEffectiveBit,
		long					nMaxFrameNum,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_MRAW16FileSaveOpen  PIC_MRAW16FileSaveOpenW
#else
#define PIC_MRAW16FileSaveOpen  PIC_MRAW16FileSaveOpenA
#endif // !UNICODE

unsigned long WINAPI
PIC_MRAW16FileSaveOpenWithA(
		PIC_MRAW				*pPicMRaw,
		LPCSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		PIC_RAW_BITDEPTH		pic_depth,
		unsigned long			nEffectiveBit,
		long					nMaxFrameNum,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MRAW16FileSaveOpenWithW(
PIC_MRAW				*pPicMRaw,
		LPCWSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		PIC_RAW_BITDEPTH		pic_depth,
		unsigned long			nEffectiveBit,
		long					nMaxFrameNum,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_MRAW16FileSaveOpenWith  PIC_MRAW16FileSaveOpenWithW
#else
#define PIC_MRAW16FileSaveOpenWith  PIC_MRAW16FileSaveOpenWithA
#endif // !UNICODE

unsigned long WINAPI
PIC_MRAW16FileSave(
		unsigned short			*pBuff,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MRAW16FileSaveWith(
		PIC_MRAW				*pPicMRaw,
		unsigned short			*pBuff,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI 
PIC_MRAWFileSaveClose(
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MRAWFileSaveCloseWith(
		PIC_MRAW				*pPicMRaw,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI 
PIC_MTIFFFileSaveOpenA(
		LPCSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		long					nCompressMode,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MTIFFFileSaveOpenW(
		LPCWSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		long					nCompressMode,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_MTIFFFileSaveOpen  PIC_MTIFFFileSaveOpenW
#else
#define PIC_MTIFFFileSaveOpen  PIC_MTIFFFileSaveOpenA
#endif // !UNICODE

unsigned long WINAPI
PIC_MTIFFFileSave(
		unsigned char			*pBuff,
		unsigned long			*pSize,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MTIFF16FileSaveOpenA(
		LPCSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		PIC_TIFF_BITDEPTH		pic_depth,
		unsigned long			nEffectiveBit,
		long					nCompressMode,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MTIFF16FileSaveOpenW(
		LPCWSTR					lpszFileName,
		PPIC_IMAGE_BUFF_INFO	pInfo,
		PIC_TIFF_BITDEPTH		pic_depth,
		unsigned long			nEffectiveBit,
		long					nCompressMode,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_MTIFF16FileSaveOpen  PIC_MTIFF16FileSaveOpenW
#else
#define PIC_MTIFF16FileSaveOpen  PIC_MTIFF16FileSaveOpenA
#endif // !UNICODE

unsigned long WINAPI
PIC_MTIFF16FileSave(
		unsigned short			*pBuff,
		unsigned long			*pSize,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MTIFFFileSaveClose(
		unsigned long			*pErrorCode
		);


/*
 *	File Load
 */
unsigned long WINAPI
PIC_RAWFileLoadA(
		LPCSTR					lpszFileName,
		PIC_IMAGE_BUFF_INFO		*pInfo,
		PIC_RAW_BITDEPTH		pic_depth,
		void					*pBuff,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_RAWFileLoadW(
		LPCWSTR					lpszFileName,
		PIC_IMAGE_BUFF_INFO		*pInfo,
		PIC_RAW_BITDEPTH		pic_depth,
		void					*pBuff,
		unsigned long			*pErrorCode
);

#ifdef UNICODE
#define PIC_RAWFileLoad  PIC_RAWFileLoadW
#else
#define PIC_RAWFileLoad  PIC_RAWFileLoadA
#endif // !UNICODE

unsigned long WINAPI
PIC_RAWFileLoad2A(
		LPCSTR					lpszFileName,
		PIC_IMAGE_BUFF_INFO		*pInfo,
		PIC_RAW_BITDEPTH		pic_depth,
		void					*pBuff,
		PIC_ORG_IMAGE_INFO		*pOrgInfo,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_RAWFileLoad2W(
		LPCWSTR					lpszFileName,
		PIC_IMAGE_BUFF_INFO		*pInfo,
		PIC_RAW_BITDEPTH		pic_depth,
		void					*pBuff,
		PIC_ORG_IMAGE_INFO		*pOrgInfo,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_RAWFileLoad2  PIC_RAWFileLoad2W
#else
#define PIC_RAWFileLoad2  PIC_RAWFileLoad2A
#endif // !UNICODE

unsigned long WINAPI
PIC_MRAWFileLoadOpenA(
		LPCSTR					lpszFileName,
		PIC_IMAGE_BUFF_INFO		*pInfo,
		PIC_MRAW				*pPicMRaw,
		PIC_RAW_BITDEPTH		pic_depth,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MRAWFileLoadOpenW(
		LPCWSTR					lpszFileName,
		PIC_IMAGE_BUFF_INFO		*pInfo,
		PIC_MRAW				*pPicMRaw,
		PIC_RAW_BITDEPTH		pic_depth,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_MRAWFileLoadOpen  PIC_MRAWFileLoadOpenW
#else
#define PIC_MRAWFileLoadOpen  PIC_MRAWFileLoadOpenA
#endif // !UNICODE

unsigned long WINAPI
PIC_MRAWFileLoadOpen2A(
		LPCSTR					lpszFileName,
		PIC_IMAGE_BUFF_INFO		*pInfo,
		PIC_MRAW				*pPicMRaw,
		PIC_RAW_BITDEPTH		pic_depth,
		PIC_ORG_IMAGE_INFO		*pOrgInfo,
		unsigned long			*pErrorCode
		);

unsigned long WINAPI
PIC_MRAWFileLoadOpen2W(
		LPCWSTR					lpszFileName,
		PIC_IMAGE_BUFF_INFO		*pInfo,
		PIC_MRAW				*pPicMRaw,
		PIC_RAW_BITDEPTH		pic_depth,
		PIC_ORG_IMAGE_INFO		*pOrgInfo,
		unsigned long			*pErrorCode
		);

#ifdef UNICODE
#define PIC_MRAWFileLoadOpen2  PIC_MRAWFileLoadOpen2W
#else
#define PIC_MRAWFileLoadOpen2  PIC_MRAWFileLoadOpen2A
#endif // !UNICODE

unsigned long WINAPI 
PIC_MRAWFileLoad(
		PIC_MRAW		*pPicMRaw,
		void			*pBuff,
		long			nFrame,
		unsigned long	*pErrorCode
		);

unsigned long WINAPI 
PIC_MRAWFileLoadClose(
		PIC_MRAW		*pPicMRaw,
		unsigned long	*pErrorCode
		);

unsigned long WINAPI 
PIC_Speeder2FileGetHeaderA(
						  LPCSTR			lpszFileName,
						  PIC_VID			*pPicVID,
						  unsigned long		*pErrorCode);

unsigned long WINAPI
PIC_Speeder2FileGetHeaderW(
						LPCWSTR			lpszFileName,
						PIC_VID			*pPicVID,
						unsigned long		*pErrorCode);

#ifdef UNICODE
#define PIC_Speeder2FileGetHeader  PIC_Speeder2FileGetHeaderW
#else
#define PIC_Speeder2FileGetHeader  PIC_Speeder2FileGetHeaderA
#endif // !UNICODE

unsigned long WINAPI 
PIC_Fil_SPV2_BayerConvert(
						  void						*pSrc,
						  void						*pDes,
						  BOOL						bReverseSrc,
						  PIC_VID					*pVIDInfo,
						  PPIC_BAYER_CONVERT_INFO	pBayerInfo,
						  unsigned long				*pErrorCode);

unsigned long WINAPI
PIC_Fil_SPV2_PixelDefRevision(void *pSrc,
							  BOOL	bReverseSrc,
							  PIC_VID *pVIDInfo,
							  unsigned long *pErrorCode);

/*
 *	Filter
 */
unsigned long WINAPI
PIC_Fil_BayerConvert(
		PPIC_IMAGE_BUFF_INFO	pInfo,
		void					*pSrc,
		void					*pDes,
		void                    *pWork,
		PPIC_BAYER_CONVERT_INFO	pBayerInfo,
		unsigned long			*pErrorCode
		);

#ifdef	__cplusplus
}
#endif	/*	__cplusplus	*/

#endif	/*	_PICLIB_H_	*/

